/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.adm;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import javax.naming.Context;

import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;
import org.objectweb.jonas.common.Trace;


/*
 * This class implements a rmi client to administrate the server.
 * Arguments are read on the command line and transfered through rmi to the server. 
 * See usage() by -?
 */
public class AdmServer { 

    public static void main(String args[]) { 

        Context    initialContext   = null;

	String     beansFileName    = null;
	String     traceOutput      = null;
	String     traceError       = null;
	String     traceLevel       = null;

	boolean    lOption          = false;
	boolean    LOption          = false;
	boolean    EOption          = false;
	boolean    aOption          = false;
	boolean    oOption          = false;
	boolean    eOption          = false;
	boolean    fOption          = false;
	boolean    hOption          = false;
	boolean    sOption          = false;
 
	Trace.configure();

	// Command line
	String commandLine = "AdmServer";
	for (int argn=0; argn<args.length; argn++) {
	    String arg = args[argn];

	    if ((argn<args.length) && (arg.equals("-l") == true)) {
		commandLine = commandLine + " -l ";
		lOption = true;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-s") == true)) {
		commandLine = commandLine + " -s ";
		sOption = true;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-a") == true)) {
		beansFileName = args[++argn];
		commandLine = commandLine + " -a "+beansFileName;
		aOption = true;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-o") == true)) {
		traceOutput = args[++argn];
		commandLine = commandLine + " -o "+traceOutput;
		oOption = true;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-e") == true)) {
		traceError = args[++argn];
		commandLine = commandLine + " -e "+traceError;
		eOption = true;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-l") == true)) {
		traceLevel = args[++argn];
		commandLine = commandLine + " -l "+traceLevel;
		fOption = true;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-L") == true)) {
		commandLine = commandLine + " -L ";
		LOption = true;
		continue;
	    }

	    if ((argn<args.length) && (arg.equals("-E") == true)) {
		commandLine = commandLine + " -E ";
		EOption = true;
		continue;
	    }

	    if ((argn<args.length) && ((arg.equals("-h") == true) || 
				       (arg.equals("-?") == true))) {
		commandLine = commandLine + " -?";
		hOption = true;
		AdmServer.usage();
	    }

	}

	Trace.outln (commandLine);

	// Checks parameters
	if (!(((oOption) && (eOption) && (fOption)) || 
	      ((!oOption) && (!eOption) && (!fOption) )))
	    usage();

	// Initializes JNDI
	if ((initialContext = Env.getInitialContext()) == null) return;


	// Gets reference on Adm object
	AdmInterface admI = null;
	{
	    String admName  = Env.getJonasName() + "_Adm";
	    try {
		admI = (AdmInterface) initialContext.lookup(admName);
	    } catch (Exception e) {
		Trace.errln ("Server.initialContext.lookup("+admName+") : " + e);
		e.printStackTrace();
		return;
	    }
	}


	////// Adds beans
	// Remote requests to Adm
	if (aOption) {
	    // Properties from file
	    Prop            fileProp  = new Prop(beansFileName);
	    String          descNames = fileProp.getValue("jonas.beans.descriptors");
	    Vector          beans     = new Vector();
	    StringTokenizer st        = new StringTokenizer(descNames, ",");

	    // Loops on beans descriptors names
	    int i = 0;
	    while(st.hasMoreTokens()) {
		beans.addElement(st.nextToken());
		i++;
	    }

	    try {
		admI.addBeans(beans);
	    } catch (RemoteException e) {
		Trace.errln ("admI.addBeans("+beans.toString()+") : "+e);
		System.exit(2);
	    }
	    System.out.println ("AdmServer -a :  Beans added in server :" + 
				beans.toString());
	}


	////// Lists beans
	// Remote request to Adm
	if (lOption) {
	    Vector result = new Vector();
	    try {
		result = admI.listBeans();
	    } catch (RemoteException e) {
		Trace.errln ("admI.listBeans() : "+e);
		System.exit(2);
	    }
	    if (result.size() == 0)
		System.out.println("AdmServer -l :  No bean in JOnAS server.");
	    for (int i=0; i<result.size(); i++) {
		System.out.println("AdmServer -l :  Bean "+i+ " in the JOnAS server = " + 
				   result.elementAt(i));
	    }
	}


	////// Lists JNDI binded references
	// Remote request to Adm
	if (LOption) {
	    Vector result = new Vector();
	    try {
		result = admI.listContext();
	    } catch (RemoteException ex) {
		Trace.errln ("admI.listBeans() : "+ex);
		System.exit(2);
	    }
	    
	    if (result.size() == 0)
		System.out.println("AdmServer -L :  No name in JNDI context.");
	    for (int i=0; i<result.size(); i++) {
		System.out.println("AdmServer -L :  Name "+i+ " in the JNDI context = " + 
				   result.elementAt(i));
	    }
	}


	////// Lists JOnAS properties 
	// Remote request to Adm
	if (EOption) {
	    Properties p = new Properties();
	    try {
		p = admI.listEnv();
	    } catch (RemoteException ex) {
		Trace.errln ("admI.listBeans() : "+ex);
		System.exit(2);
	    }
	    
	    for (Enumeration e = p.keys(); e.hasMoreElements();) {
		Object key   = e.nextElement();
		Object value = p.get(key);
		System.out.println("AdmServer -E :  "+key.toString() + "=" + value.toString());
	    }	
	}


	////// Configures JOnAS traces 
	// Remote request to Adm
	if (((oOption) && (eOption) && (fOption))) {
	    try {
		admI.configureTrace (traceOutput, traceError, traceLevel);
	    } catch (RemoteException e) {
		Trace.errln ("admI.configureTrace " + 
			     traceOutput + " " + 
			     traceError  + " " + 
			     traceLevel  + " : " + e);
	    }
	    System.out.println("AdmServer -o -e -f :  Server traces set to <output="+ 
			       traceOutput + "> <error=" + 
			       traceError  + "> <flags=" + 
			       traceLevel  + ">");
	}

	////// Stops Server
	if(sOption){
	    try{
		admI.stopServer();
	    }catch (RemoteException ex) {
		System.exit(0);
	    }	
	}
    }
    

    
    static void usage() {
	System.err.println (" usage : AdmServer [ -s ] [ -l ] [ -L ] [ -E ]");
	System.err.println ("                   [ -a  BeansFileName ] ");
	System.err.println ("                   [ -o Output -e Error -f {<flag1>,<flag2>,<flag3>,...} ]");
	System.err.println ("                   [ -h | -? ]" );
	System.err.println ("" );
	System.err.println ("    -s : stops the EJB server.");
	System.err.println ("    -l : lists types of beans currently in the EJB Server.");
	System.err.println ("    -L : lists registered JNDI names.");
	System.err.println ("    -E : lists JOnAS properties currently used by the EJB Server.");
	System.err.println ("    -a BeansFileName : dynamically adds types of beans in the EJB Server.");
	System.err.println ("    -h : prints this message.");
	System.err.println ("    -? : prints this message.");
	System.exit(2);
    }
}
