/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.container;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Properties;
import javax.ejb.EJBObject;
import javax.ejb.HomeHandle;
import org.objectweb.jonas.common.Trace;


/**
 * This class implements javax.ejb.Handle interface.
 * For a Entity Bean  a Handle is a serializable class
 * that contains the HomeHandle and the primary key
 */
abstract public class EntityHandleImpl implements javax.ejb.Handle, java.io.Serializable {

    /**
     * @serial
     */
    protected HomeHandle homehandle = null;

    /**
     * @serial
     */
    protected Serializable pk = null;

    /**
     * constructor
     */    
    public EntityHandleImpl(JBeanEntity ejbobject) {
	try {
	    homehandle = ejbobject.getHome().getHomeHandle();
	} catch (RemoteException e) {
	    Trace.errln("Cannot get HomeHandle: "+e);
	}
    }

    
    /** 
     * Obtains the EJB object represented by this handle. 
     * @Throws RemoteException: The EJB object could not be obtained because of a system-level
     * failure. 
     * Properties java.naming.factory.initial & java.naming.provider.url must be set
     * in the System properties.
     * Example: -Djava.naming.factory.initial=com.sun.jndi.rmi.registry.RegistryContextFactory
     *          -Djava.naming.provider.url=rmi://hostname:1099
     */
   abstract public EJBObject getEJBObject() throws java.rmi.RemoteException;

}

