/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import java.util.Properties;
import javax.ejb.EJBHome;
import javax.naming.Context;
import javax.naming.InitialContext;


/**
 * This class implements javax.ejb.HomeHandle interface.
 * A handle is an abstraction of a network reference to a home object.
 * A handle is intended to be used as a "robust" persistent
 * reference to a home object. 
 * SINCE EJB 1.1 only.
 * 
 */
public class HomeHandleImpl implements javax.ejb.HomeHandle, java.io.Serializable {
    
    /**
     * @serial
     */
    private String homename = null;

    /**
     * @serial
     */
    private Properties jndiprop = null;


    /**
     * constructor. It's called from the EJBHome by getHomeHandle().
     */    
    public HomeHandleImpl(String hname, Properties jndi) {
	homename = hname;
	jndiprop = jndi;
    }
    

    /** 
     * Obtains the home object represented by this handle. 
     * @Throws RemoteException: The home object could not be obtained because of a 
     * system-level failure. 
     */
    public EJBHome getEJBHome() throws java.rmi.RemoteException {
	EJBHome ejbh = null;
	try {
	    Context ic = new InitialContext(jndiprop);
	    ejbh = (EJBHome) ic.lookup(homename);
	} catch (javax.naming.NamingException e) {
	    throw new java.rmi.RemoteException("getEJBHome:"+e);
	}
	return ejbh;
    }
}
