/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import java.util.Properties;
import javax.ejb.SessionBean;
import javax.ejb.EJBContext;
import javax.ejb.EJBHome;
import javax.ejb.EJBObject;
import javax.ejb.SessionContext;
import javax.transaction.Transaction;
import javax.transaction.UserTransaction;
import org.objectweb.jonas.common.Trace;


/**
 * This class implements javax.ejb.SessionContext interface.
 */
public class SessionContextImpl extends EJBContextImpl implements SessionContext {

    static private final int trace = Trace.DB_12;

    private JBeanSession ejbObject = null; // the remote object
    private SessionBean sessionBean = null; // the bean instance

    Transaction beanAssociatedTransaction = null;  // Transaction currently associated with the bean instance
    SessionSynchroImpl synchro = null; // Synchro associated with the bean instance
    Transaction clientTransaction = null; // Tx to be resumed at postinvoke
    boolean mustBeCommitted = false; // must commit current Tx at postinvoke

    // ------------------------------------------------------------------
    // constructors
    // ------------------------------------------------------------------

    /**
     * Constructs a SessionContext 
     * 
     * @param obj the EJB Object created
     * @param curr the UserTransaction object
     * @param env  the environment properties
     * @param home enterprise bean's home interface 
     *
     */
    public SessionContextImpl(JBeanSession obj, 
			      JBeanHome home, 
			      SessionBean sb,
			      SessionSynchroImpl sync) {
	super(home);
	Trace.outln(trace,"SessionContextImpl.SessionContextImpl(EJBObject "+obj+" , EJBHome " +home);
	ejbObject = obj;
	sessionBean = sb;
	synchro = sync;
    }


    // ------------------------------------------------------------------
    // SessionContext implementation
    // ------------------------------------------------------------------


    /**
     * Obtains a reference to the EJB object that is currently associated with the instance. 
     * An instance of a session enterprise Bean can call this method at anytime between the
     *  ejbCreate() and ejbRemove() methods, including from within the ejbCreate() and
     *  ejbRemove() methods.
     * An instance can use this method, for example, when it wants to pass a reference 
     *  to itself in a method argument or result.
     *
     * @return    The EJB object currently associated with the instance. 
     * @exception IllegalStateException: Thrown if the instance invokes this method while 
     *            the instance is in a state that does not allow the instance to invoke 
     *            this method.
     */
    public  EJBObject getEJBObject() throws IllegalStateException {
	Trace.outln(trace,"SessionContextImpl.getEJBObject()");
	return ejbObject;
    }

    // ------------------------------------------------------------------
    // Other methods
    // ------------------------------------------------------------------

    /**
     * Returns the bean instance of this context
     *
     * @return the bean instance
     *
     */
    public SessionBean getInstance() {
	return sessionBean;
    }
}
