/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.ldap;

import javax.naming.directory.DirContext;
import javax.naming.NamingException;

/**
 * A DataSourceLDAP object is a factory for JNDI DirContext objects. 
 * This object mimics the DataSource object used for databases.
 */
public interface DataSourceLDAP {

    /**
     * Set the list of byte-array attributes for this bean.
     * Note that we have one datasource per lookup (i.e. per bean)!
     *
     * @param name of the attribute
     */
    public void setBinAttribs(String v) throws NamingException;

    /**
     * Get a new DirContext
     *
     * @return the DirContext
     */
    public DirContext getDirContext() throws NamingException;

    /**
     * Get a new DirContext for this user
     *
     * @param user - the user on whose behalf the DirContext will be used
     * @param password - the user's password
     *
     * @return the DirContext
     */
    public DirContext getDirContext(String user, String passwd) throws NamingException;
}
