/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): Vincent Trussart (vincent@linuxfreak.com)
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/**
 * Contains static methods to marshall/unmarshall objects to/from a byte[]
 *
 * Created: Thu Aug 12 16:49:04 1999
 */

package org.objectweb.jonas.lib;


import java.io.*;


public class MarshallTool  {

    /**
     * ByteArrayOutpuStream internal buffer size.  
     */   
    private static int _buffSize = 4096;

    /**
     * Converts an instance of java.io.Serializable to a serialized byte array
     *
     * @param o Object to serialize
     * @return byte array containing the serialization of the Serializable object
     */
    public static synchronized byte[] toBytes(Serializable o) throws IOException {

	ByteArrayOutputStream bas = new ByteArrayOutputStream(_buffSize);
	ObjectOutputStream oos = new ObjectOutputStream(bas);

	oos.writeObject(o);
	oos.close();

	return bas.toByteArray();
    }

    /**
     * Converts byte array to an instance of java.io.Serializable
     *
     * @param bytes A byte array containing the a serialized object
     * @return the unmarshalled object
     */
    public static synchronized Serializable fromBytes(byte[] bytes) throws IOException, ClassNotFoundException {

	if (bytes == null) return null;

	ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
	ObjectInputStream ois = new ObjectInputStream(bis);

	return (Serializable)ois.readObject();
    }
    
} // MarshallTool

