/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.rmifilters;

import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;
import java.rmi.MarshalException;
import java.rmi.UnmarshalException;

import org.objectweb.jonas.jta.Current;
import org.objectweb.jonas.jta.TransactionImpl;
import org.objectweb.jonas.jtm.PropagationContext;

/**
 * The RemoteStub class is the common superclass to all client stubs
 * generated by the Bull rmic compiler used for the implicit transactional
 * context propagation.
 *
 * invoke() inserts in the ObjectOutputStream the Control object corresponding
 * to the current transaction.
 */

public class RemoteStub extends java.rmi.server.RemoteStub {

    /**
     * Default constructor
     */
    public RemoteStub() {
	super();
    }


    /**
     * Constructor for RemoteStub, with the specified remote reference. 
     */
    public RemoteStub(java.rmi.server.RemoteRef ref) {
	super(ref);
    }


    /**
     * invoke() inserts the correct transactional context before executing the remote call.
     * A null object means "no transactional context".
     */
    public void invoke(java.rmi.server.RemoteCall call) 
	throws java.lang.Exception, java.rmi.MarshalException {
	
	// Get the Propagation Context
	PropagationContext pctx = null;
	Current ut = Current.getCurrent();
	if (ut != null) {
	    pctx = ut.getPropagationContext();
	}

	// Add it to output args
	try {
	    ObjectOutput out = call.getOutputStream();
	    out.writeObject(pctx);
	} catch (IOException e) {
	    throw new MarshalException("error marshalling propagation context", e);
	}

	// Invoke remote method call
	ref.invoke(call);

        // Extracts PropagationContext from input args
        try {
            ObjectInput in = call.getInputStream();
	    pctx = (PropagationContext) in.readObject();
	} catch (IOException e) {
	    throw new UnmarshalException("error unmarshalling propagation context", e);
	}	

	// Associates the current thread with this propagation context
        if (pctx != null && ut != null) {
	    ut.setPropagationContext(pctx);
        }
    }
}
