/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.lang.String;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.BeanNaming;
import org.objectweb.jonas.lib.DD;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Entity Enterprise Java Bean
 *  with container managed persistence by JDBC.
 *  Its extends GenICEntityCMHome class, itself extends GenICEntityHome class,
 *  itself extends GenICHome.
 */
class GenICEntityCMbyJDBCHome extends GenICEntityCMHome {

    String dbTableName = null;

    /**
     * GenICEntityCMbyJDBCHome Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     *                 with container managed persistence by JDBC
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityCMbyJDBCHome(DD dd, String fileName) throws GenICException {
	super(dd, fileName);
	// Get the database table name
	try {
	    dbTableName = ddesc.getDBTableName();
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.Object;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import java.sql.Connection;");
	src.println("import java.sql.PreparedStatement;");
	src.println("import java.sql.ResultSet;");
	src.println("import java.sql.SQLException;");
	src.println("import java.util.Enumeration;");
	src.println("import javax.ejb.EJBObject;");
	src.println("import javax.ejb.FinderException;");
	src.println("import javax.ejb.ObjectNotFoundException;");
	src.println("import javax.ejb.deployment.EntityDescriptor;");
	src.println("import javax.sql.DataSource;");
	src.println("import org.objectweb.jonas.container.JEntityHome;");
	src.println("import org.objectweb.jonas.container.EntityContextImpl;");
	src.println("import org.objectweb.jonas.lib.CollectionEnum;");
	src.println("import org.objectweb.jonas.common.Trace;");
    }

    /**
     * Source generation of the ejbFindByPrimaryKey() method associated to the given
     * findByPrimaryKey() method described in the Home interface
     * @exception GenICException in error case
     */
    void genEjbFindByPK(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();

	src.print  ("synchronized private " + ejbPrimaryKeyName + " pk" +
		    BeanNaming.firstToUpperCase(method.getName()) +
		    "(");
	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]) + " " + "p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.print  (") ", false);
	src.println(" throws java.rmi.RemoteException, javax.ejb.FinderException {", false);
	// core
	src.indentPlus();
	src.print  ("Trace.outln(Trace.DB_15, \""+wrpHomeBaseName+".pk"+
		    BeanNaming.firstToUpperCase(method.getName())+"(");
	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]), false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(")\");", false);
	src.println("Connection conn = null;");
	src.println("PreparedStatement pStmt = null;");
	src.println("try {");
	src.indentPlus();
	src.println("conn = ((DataSource)getDataSource()).getConnection();");
	src.print  ("pStmt = conn.prepareStatement(\"select ");
	try {
	    for (int i=0; i<fieldsPK.length; i++) {
		if (i > 0) {
		    src.print(", ", false);
		}
		src.print(ddesc.getDBFieldName(fieldsPK[i]), false);
	    }
	    src.print (" from " + dbTableName + " where ", false);
	    for (int i=0; i<fieldsPK.length; i++) {
		if (i > 0) {
		    src.print(" and ", false);
		}
		src.print(ddesc.getDBFieldName(fieldsPK[i]) +
			  "=?", false);
	    }
	    src.println("\");", false);
	    for (int i=0; i<fieldsPK.length; i++) {
		int index = i + 1;
		String methName = JavaType.getSQLSetMethod(fieldsPK[i].getType());
		String attrName = "p1."+fieldsPK[i].getName();
		String castS;
		if (JavaType.isXxxObjectMethod(methName)) {
		    castS = "(java.lang.Object)";
		} else {
		    castS = "";
		}
		src.println("pStmt."+ methName +
			    "(" + index + ", " + castS + attrName + ");");
	    }
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
	src.println("ResultSet rs = pStmt.executeQuery();");
	src.println("if (rs.next() == false) {");
	src.indentPlus();
	src.println("Trace.errln(\"Object not found in database ("+
		    method.getName()+")\");");
	src.println("throw new ObjectNotFoundException(\"Object not found in database ("+
		    method.getName()+")\");");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("} catch (SQLException e) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to find bean from database ("+
		    method.getName()+")\");");
	src.println("Trace.errln(e);");
	src.println("throw new FinderException(\"Failed to find bean from database ("+
		    method.getName()+")\");");
	src.indentMinus();
	src.println("} finally {");
	src.indentPlus();
	src.println("if (pStmt != null) {");
	src.indentPlus();
	src.println("try {");
	src.indentPlus();
	src.println("pStmt.close();");
	src.indentMinus();
	src.println("} catch (Exception ignore) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to close the PreparedStatement ("+
		    method.getName()+")\");");
	src.println("Trace.errln(ignore);");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}");
	src.println("if (conn != null) {");
	src.indentPlus();
	src.println("try {");
	src.indentPlus();
	src.println("conn.close();");
	src.indentMinus();
	src.println("} catch (Exception ignore) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to close the Connection ("+
		    method.getName()+")\");");
	src.println("Trace.errln(ignore);");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}"); // try-catch-finally
	src.println("return(p1);");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation of the ejbFindXXX() method associated to the given
     * findXXX() method described in the Home interface
     * @exception GenICException in error case
     */
    void genEjbFindOther(Method method) throws GenICException {

	boolean isWithCollection = ! method.getReturnType().equals(ejbRemoteClass);
	Class params[] = method.getParameterTypes();

	if (isWithCollection) {
	    src.print  ("synchronized private CollectionEnum" + 
			" pk" + BeanNaming.firstToUpperCase(method.getName()) +
			"(");
	} else {
	    src.print  ("synchronized private " + ejbPrimaryKeyName +
			" pk" + BeanNaming.firstToUpperCase(method.getName()) +
			"(");
	}
	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]) + " " + "p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.print  (") ", false);
	src.println(" throws java.rmi.RemoteException, javax.ejb.FinderException {", false);
	// core
	src.indentPlus();
	src.print  ("Trace.outln(Trace.DB_15, \""+wrpHomeBaseName+".pk"+
		    BeanNaming.firstToUpperCase(method.getName())+"(");
	for (int p=1; p <= params.length; p++) {
	    src.print(JavaType.getName(params[p-1]), false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(")\");", false);
	if (isWithCollection) {
	    src.println("CollectionEnum pkC = new CollectionEnum();");
	} else {
	    src.println(ejbPrimaryKeyName + " pk = " +
			"new " + ejbPrimaryKeyName + "();");
	}
	src.println("Connection conn = null;");
	src.println("PreparedStatement pStmt = null;");
	src.println("try {");
	src.indentPlus();
	src.println("conn = ((DataSource)getDataSource()).getConnection();");
	src.print  ("pStmt = conn.prepareStatement(\"select ");
	try {
	    for (int i=0; i<fieldsPK.length; i++) {
		if (i > 0) {
		    src.print(", ", false);
		}
		src.print(ddesc.getDBFieldName(fieldsPK[i]), false);
	    }
	    src.println(" from " + dbTableName + " " +
			ddesc.getDBFinderWhereClause(method) + "\");",
			false);
	    for (int i=1; i <= params.length; i++) {
		String methName = JavaType.getSQLSetMethod(params[i-1]);
		String attrName = new String("p" + i);
		if (methName == null) {
		    throw new GenICException("Cannot container manage the bean's method '" +
					     method +
					     "' (Param '" + JavaType.getName(params[i-1]) + "')");
		}
		String castS;
		if (JavaType.isXxxObjectMethod(methName)) {
		    castS = "(java.lang.Object)";
		} else {
		    castS = "";
		}
		src.println("pStmt."+ methName +
			    "(" + i + ", " + castS + attrName + ");");
	    }
	    src.println("ResultSet rs = pStmt.executeQuery();");
	    if (isWithCollection) {
		src.println("while (rs.next()) {");
		src.indentPlus();
		src.println(ejbPrimaryKeyName + " pk = " +
			    "new " + ejbPrimaryKeyName + "();");
		for (int i=0; i<fieldsPK.length; i++) {
		    String methName = JavaType.getSQLGetMethod(fieldsPK[i].getType());
		    String attrName = "pk."+fieldsPK[i].getName();
		    String dbclName = ddesc.getDBFieldName(fieldsPK[i]);
		    if (methName == null) {
			throw new GenICException("Cannot container manage the bean's method '" +
						 method +
						 "' (Param '" + JavaType.getName(params[i-1]) + "')");
		    }
		    String castS;
		    if (JavaType.isXxxObjectMethod(methName)) {
			castS = "("+JavaType.getName(fieldsPK[i].getType())+")";
		    } else {
			castS = "";
		    }
		    src.println(attrName + " = " + castS + "rs." + methName +
				"(\"" + dbclName + "\");");
		}
		src.println("pkC.addElement((Object)pk);");
		src.indentMinus();
		src.println("}");
	    } else {
		src.println("if (rs.next() == false) {");
		src.indentPlus();
		src.println("Trace.errln(\"Object not found in database ("+
			    method.getName()+")\");");
		src.println("throw new ObjectNotFoundException(\"Object not found in database ("+
			    method.getName()+")\");");
		src.indentMinus();
		src.println("}");
		for (int i=0; i<fieldsPK.length; i++) {
		    String methName = JavaType.getSQLGetMethod(fieldsPK[i].getType());
		    String attrName = "pk."+fieldsPK[i].getName();
		    String dbclName = ddesc.getDBFieldName(fieldsPK[i]);
		    String castS;
		    if (JavaType.isXxxObjectMethod(methName)) {
			castS = "("+JavaType.getName(fieldsPK[i].getType())+")";
		    } else {
			castS = "";
		    }
		    src.println(attrName + " = " + castS + "rs." + methName +
				"(\"" + dbclName + "\");");
		}
	    }
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
	src.indentMinus();
	src.println("} catch (SQLException e) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to find bean from database ("+
		    method.getName()+")\");");
	src.println("Trace.errln(e);");
	src.println("throw new FinderException(\"Failed to find bean from database ("+
		    method.getName()+")\");");
	src.indentMinus();
	src.println("} finally {");
	src.indentPlus();
	src.println("if (pStmt != null) {");
	src.indentPlus();
	src.println("try {");
	src.indentPlus();
	src.println("pStmt.close();");
	src.indentMinus();
	src.println("} catch (Exception ignore) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to close the PreparedStatement ("+
		    method.getName()+")\");");
	src.println("Trace.errln(ignore);");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}");
	src.println("if (conn != null) {");
	src.indentPlus();
	src.println("try {");
	src.indentPlus();
	src.println("conn.close();");
	src.indentMinus();
	src.println("} catch (Exception ignore) {");
	src.indentPlus();
	src.println("Trace.errln(\"Failed to close the connection ("+
		    method.getName()+")\");");
	src.println("Trace.errln(ignore);");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}"); // try-catch-finally
	if (isWithCollection) {
	    src.println("return(pkC);");
	} else {
	    src.println("return(pk);");
	}
	src.indentMinus();
	src.println("}");
    }

}



