/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;

import java.lang.String;
import java.lang.reflect.Field;
import org.objectweb.jonas.lib.DD;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Entity Enterprise Java Bean
 *  with container managed persistence on JDBC.
 *  Its extends GenICEntityCMRemote, itself extends GenICEntityRemote class,
 *  itself extends GenICRemote class.
 */
class GenICEntityCMbyJDBCRemote extends GenICEntityCMRemote {

    // Database Table Name
    String  dbTableName    = null;

    /**
     * GenICEntityCMbyJDBCRemote Constructor
     *
     * @param dd       deployment descriptor of the Entity Bean with container managed persistence
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityCMbyJDBCRemote(DD dd, String fileName) throws GenICException {

	super(dd, fileName);

	// Get the database table name
	try {
	    dbTableName = ddesc.getDBTableName();
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import java.sql.Connection;");
	src.println("import java.sql.PreparedStatement;");
	src.println("import java.sql.ResultSet;");
	src.println("import java.sql.SQLException;");
	src.println("import java.sql.Types;");
	src.println("import javax.ejb.EnterpriseBean;");
	src.println("import javax.ejb.RemoveException;");
	src.println("import javax.ejb.EntityBean;");
	src.println("import javax.sql.DataSource;");
	src.println("import org.objectweb.jonas.container.JEntityHome;");
	src.println("import org.objectweb.jonas.container.JBeanEntity;");
	src.println("import org.objectweb.jonas.container.EntitySynchroImpl;");
	src.println("import org.objectweb.jonas.container.SynchroImpl;");
	src.println("import org.objectweb.jonas.lib.MarshallTool;");
	src.println("import org.objectweb.jonas.common.Trace;");
    }

    /**
     * Source generation of the createData(), removeData(), loadData(), storeData() methods
     * The isModifiedData() method is generated en the Super Class.
     * @exception GenICException in error case
     */
    void genAccessData() throws GenICException {
	super.genAccessData();
	genCreateData();
	src.println();
	genRemoveData();
	src.println();
	genLoadData();
	src.println();
	genStoreData();
	src.println();
    }

    /**
     * Source generation of the createData() method
     * @exception GenICException in error case
     */
    private void genCreateData() throws GenICException {
	try {
	    src.println("public void createData() throws RemoteException {");
	    src.indentPlus();
	    src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".createData()\");");
	    src.println("Connection conn = null;");
	    src.println("PreparedStatement pStmt = null;");
	    src.println("try {");
	    src.indentPlus();
	    src.println(ejbBeanName + " bn = ("+ejbBeanName+")getBean();");
	    src.println("conn = ((DataSource)(getHome().getDataSource())).getConnection();");
	    src.print  ("pStmt = conn.prepareStatement(\"insert into " + dbTableName +" (");
	    for (int i=0; i<fieldsPers.length; i++) {
		if (i > 0) {
		    src.print(", ", false);
		}
		src.print(ddesc.getDBFieldName(fieldsPers[i]), false);
	    }
	    src.print  (") values (", false);
	    for (int i=0; i<fieldsPers.length; i++) {
		if (i > 0) {
		    src.print(", ", false);
		}
		src.print("?", false);
	    }
	    src.println(")\");", false);
	    for (int i=0; i<fieldsPers.length; i++) {
		int index = i+1;
		Class  attrType = fieldsPers[i].getType();
		String sqlTypeName = JavaType.getSQLType(attrType);
		String methName = JavaType.getSQLSetMethod(attrType);
		String attrName = "bn."+fieldsPers[i].getName();
		String castS;
		if (JavaType.isXxxObjectMethod(methName)) {
		    castS = "(java.lang.Object)";
		} else {
		    castS = "";
		}
		if (! attrType.isPrimitive()) {
		    src.println("if ("+attrName+" == null) {");
		    src.indentPlus();
		    src.println("pStmt.setNull("+ index + ", " + sqlTypeName + ");");
		    src.indentMinus();
		    src.println("} else {");
		    src.indentPlus();
		}
		if (methName.equals("setSerializable")){
		    src.println("pStmt.setBytes(" + 
				index + ", MarshallTool.toBytes(" + attrName + "));");
		} else {
		    src.println("pStmt." + methName + "(" + 
				index + ", " + castS + attrName + ");");
		}
		if (! attrType.isPrimitive()) {
		    src.indentMinus();
		    src.println("}");
		}
	    }
	    src.println("pStmt.executeUpdate();");
	    src.indentMinus();
	    //src.println("} catch (SQLException e) {");
	    src.println("} catch (Exception e) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to create bean in database\");");
	    src.println("Trace.errln(e);");
	    src.println("throw new RemoteException(\"Failed to create bean in database\", e);");
	    src.indentMinus();
	    src.println("} finally {");
	    src.indentPlus();
	    src.println("if (pStmt != null) {");
	    src.indentPlus();
	    src.println("try {");
	    src.indentPlus();
	    src.println("pStmt.close();");
	    src.indentMinus();
	    src.println("} catch (Exception ignore) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to close the PreparedStatement (CreateData)\");");
	    src.println("Trace.errln(ignore);");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}");
	    src.println("if (conn != null) {");
	    src.indentPlus();
	    src.println("try {");
	    src.indentPlus();
	    src.println("conn.close();");
	    src.indentMinus();
	    src.println("} catch (Exception ignore) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to close the Connection (CreateData)\");");
	    src.println("Trace.errln(ignore);");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}"); // try-catch-finally
	    src.indentMinus();
	    src.println("}");
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
    }

    /**
     * Source generation of the removeData() method
     * @exception GenICException in error case
     */
    private void genRemoveData() throws GenICException {
	try {
	    src.println("public void removeData() throws RemoteException {");
	    src.indentPlus();
	    src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".removeData()\");");
	    src.println("Connection conn = null;");
	    src.println("PreparedStatement pStmt = null;");
	    src.println("try {");
	    src.indentPlus();
	    src.println(ejbPrimaryKeyName+" pk = ("+ejbPrimaryKeyName+")getPrimaryKey();");
	    src.println("conn = ((DataSource)(getHome().getDataSource())).getConnection();");
	    src.print  ("pStmt = conn.prepareStatement(\"delete from " + dbTableName + " where ");
	    for (int i=0; i<fieldsPK.length; i++) {
		if (i > 0) {
		    src.print(" and ", false);
		}
		src.print(ddesc.getDBFieldName(fieldsPK[i]) + "=?", false);
	    }
	    src.println("\");", false);
	    for (int i=0; i<fieldsPK.length; i++) {
		int index = i+1;
		String methName = JavaType.getSQLSetMethod(fieldsPK[i].getType());
		String attrName = "pk."+fieldsPK[i].getName();
		String castS;
		if (JavaType.isXxxObjectMethod(methName)) {
		    castS = "(java.lang.Object)";
		} else {
		    castS = "";
		}
		src.println("pStmt." + methName +
			    "(" + index + ", " + castS + attrName + ");");
	    }
	    src.println("pStmt.executeUpdate();");
	    src.indentMinus();
	    //src.println("} catch (SQLException e) {");
	    src.println("} catch (Exception e) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to delete bean from database\");");
	    src.println("Trace.errln(e);");
	    src.println("throw new RemoteException(\"Failed to delete bean from database\", e);");
	    src.indentMinus();
	    src.println("} finally {");
	    src.indentPlus();
	    src.println("if (pStmt != null) {");
	    src.indentPlus();
	    src.println("try {");
	    src.indentPlus();
	    src.println("pStmt.close();");
	    src.indentMinus();
	    src.println("} catch (Exception ignore) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to close the PreparedStatement (RemoveData)\");");
	    src.println("Trace.errln(ignore);");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}");
	    src.println("if (conn != null) {");
	    src.indentPlus();
	    src.println("try {");
	    src.indentPlus();
	    src.println("conn.close();");
	    src.indentMinus();
	    src.println("} catch (Exception ignore) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to close the Connection (RemoveData)\");");
	    src.println("Trace.errln(ignore);");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}"); // try-catch-finally
	    src.indentMinus();
	    src.println("}");
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
    }

    /**
     * Source generation of the loadData() method
     * @exception GenICException in error case
     */
    private void genLoadData() throws GenICException {
	try {
	    src.println("public void loadData() throws RemoteException {");
	    src.indentPlus();
	    src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".loadData()\");");
	    src.println("Connection conn = null;");
	    src.println("PreparedStatement pStmt = null;");
	    src.println("try {");
	    src.indentPlus();
	    src.println(ejbBeanName + " bn = ("+ejbBeanName+")getBean();");
	    src.println(ejbPrimaryKeyName+" pk = ("+ejbPrimaryKeyName+")getPrimaryKey();");
	    src.println("conn = ((DataSource)(getHome().getDataSource())).getConnection();");
	    src.print  ("pStmt = conn.prepareStatement(\"select ");
	    for (int i=0; i<fieldsPers.length; i++) {
		if (i > 0) {
		    src.print(", ", false);
		}
		src.print(ddesc.getDBFieldName(fieldsPers[i]), false);
	    }
	    src.print (" from " + dbTableName + " where ", false);
	    for (int i=0; i<fieldsPK.length; i++) {
		if (i > 0) {
		    src.print(" and ", false);
		}
		src.print(ddesc.getDBFieldName(fieldsPK[i]) + "=?", false);
	    }
	    src.println("\");", false);
	    for (int i=0; i<fieldsPK.length; i++) {
		int index = i+1;
		String methName = JavaType.getSQLSetMethod(fieldsPK[i].getType());
		String attrName = "pk."+fieldsPK[i].getName();
		String castS;
		if (JavaType.isXxxObjectMethod(methName)) {
		    castS = "(java.lang.Object)";
		} else {
		    castS = "";
		}
		src.println("pStmt."+ methName +
			    "(" + index + ", " + castS + attrName + ");");
	    }
	    src.println("ResultSet rs = pStmt.executeQuery();");
	    src.println("if (rs.next() == false) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to load bean from database\");");
	    src.println("throw new RemoteException(\"Failed to load bean from database\");");
	    src.indentMinus();
	    src.println("}");
	    for (int i=0; i<fieldsPers.length; i++) {
		String methName = JavaType.getSQLGetMethod(fieldsPers[i].getType());
		String attrName = "bn."+fieldsPers[i].getName();
		String dbclName = ddesc.getDBFieldName(fieldsPers[i]);
		String castS;
		if (JavaType.isXxxObjectMethod(methName)) {
		    castS = "("+JavaType.getName(fieldsPers[i].getType())+")";
		} else {
		    castS = "";
		}
		if (methName.equals("getSerializable")){
		    castS = "(" + fieldsPers[i].getType().getName() + ")";
		    src.println(attrName + " = " + castS +
				"MarshallTool.fromBytes((byte[])rs.getBytes(\"" + dbclName + "\"));");
		} else {
		    src.println(attrName + " = " + castS + "rs." + methName +
				"(\"" + dbclName + "\");");
		}
	    }
	    src.indentMinus();
	    //src.println("} catch (SQLException e) {");
	    src.println("} catch (Exception e) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to load bean from database\");");
	    src.println("Trace.errln(e);");
	    src.println("throw new RemoteException(\"Failed to load bean from database\", e);");
	    src.indentMinus();
	    src.println("} finally {");
	    src.indentPlus();
	    src.println("if (pStmt != null) {");
	    src.indentPlus();
	    src.println("try {");
	    src.indentPlus();
	    src.println("pStmt.close();");
	    src.indentMinus();
	    src.println("} catch (Exception ignore) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to close the PreparedStatement (LoadData)\");");
	    src.println("Trace.errln(ignore);");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}");
	    src.println("if (conn != null) {");
	    src.indentPlus();
	    src.println("try {");
	    src.indentPlus();
	    src.println("conn.close();");
	    src.indentMinus();
	    src.println("} catch (Exception ignore) {");
	    src.indentPlus();
	    src.println("Trace.errln(\"Failed to close the Connection (LoadData)\");");
	    src.println("Trace.errln(ignore);");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}");
	    src.indentMinus();
	    src.println("}"); // try-catch-finally
	    src.indentMinus();
	    src.println("}");
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
    }

    /**
     * Source generation of the storeData() method
     * @exception GenICException in error case
     */
    private void genStoreData() throws GenICException {
	try {
	    src.println("public void storeData() throws RemoteException {");
	    src.indentPlus();
	    src.println("Trace.outln(Trace.DB_15, \"" + wrpRemoteBaseName + ".storeData()\");");
	    if (fieldsPersNoPK.length != 0) {
		src.println("Connection conn = null;");
		src.println("PreparedStatement pStmt = null;");
		src.println("try {");
		src.indentPlus();
		src.println(ejbBeanName + " bn = ("+ejbBeanName+")getBean();");
		src.println(ejbPrimaryKeyName+" pk = ("+ejbPrimaryKeyName+")getPrimaryKey();");
		src.println("conn = ((DataSource)(getHome().getDataSource())).getConnection();");
		src.print  ("pStmt = conn.prepareStatement(\"update " + dbTableName + " set ");
		for (int i=0; i<fieldsPersNoPK.length; i++) {
		    if (i > 0) {
			src.print(", ", false);
		    }
		    src.print(ddesc.getDBFieldName(fieldsPersNoPK[i]) + "=?", false);
		}
		src.print (" where ", false);
		for (int i=0; i<fieldsPK.length; i++) {
		    if (i > 0) {
			src.print(" and ", false);
		    }
		    src.print(ddesc.getDBFieldName(fieldsPK[i]) + "=?", false);
		}
		src.println("\");", false);
		for (int i=0; i<fieldsPersNoPK.length; i++) {
		    int index = i+1;
		    Class  attrType = fieldsPersNoPK[i].getType();
		    String sqlTypeName = JavaType.getSQLType(attrType);
		    String methName = JavaType.getSQLSetMethod(attrType);
		    String attrName = "bn."+fieldsPersNoPK[i].getName();
		    String castS;
		    if (JavaType.isXxxObjectMethod(methName)) {
			castS = "(java.lang.Object)";
		    } else {
			castS = "";
		    }
		    if (! attrType.isPrimitive()) {
			src.println("if ("+attrName+" == null) {");
			src.indentPlus();
			src.println("pStmt.setNull("+ index + ", " + sqlTypeName + ");");
			src.indentMinus();
			src.println("} else {");
			src.indentPlus();
		    }
		    if (methName.equals("setSerializable")){
			src.println("pStmt.setBytes(" + index + ", MarshallTool.toBytes(" + attrName + "));");
		    } else {
			src.println("pStmt." + methName +
				    "(" + index + ", " + castS + attrName + ");");		
		    }
		    if (! attrType.isPrimitive()) {
			src.indentMinus();
			src.println("}");
		    }
		}
		for (int i=0; i<fieldsPK.length; i++) {
		    int index = fieldsPersNoPK.length+i+1;
		    Class  attrType = fieldsPK[i].getType();
		    String sqlTypeName = JavaType.getSQLType(attrType);
		    String methName = JavaType.getSQLSetMethod(attrType);
		    String attrName = "pk."+fieldsPK[i].getName();
		    String castS;
		    if (JavaType.isXxxObjectMethod(methName)) {
			castS = "(java.lang.Object)";
		    } else {
			castS = "";
		    }
		    if (! attrType.isPrimitive()) {
			src.println("if ("+attrName+" == null) {");
			src.indentPlus();
			src.println("pStmt.setNull("+ index + ", " + sqlTypeName + ");");
			src.indentMinus();
			src.println("} else {");
			src.indentPlus();
		    }
		    src.println("pStmt." + methName +
				"(" + index + ", " + castS + attrName + ");");
		    if (! attrType.isPrimitive()) {
			src.indentMinus();
			src.println("}");
		    }
		}
		src.println("pStmt.executeUpdate();");
		src.indentMinus();
		//src.println("} catch (SQLException e) {");
		src.println("} catch (Exception e) {");
		src.indentPlus();
		src.println("Trace.errln(\"Failed to store bean to database\");");
		src.println("Trace.errln(e);");
		src.println("throw new RemoteException(\"Failed to store bean to database\", e);");
		src.indentMinus();
		src.println("} finally {");
		src.indentPlus();
		src.println("if (pStmt != null) {");
		src.indentPlus();
		src.println("try {");
		src.indentPlus();
		src.println("pStmt.close();");
		src.indentMinus();
		src.println("} catch (Exception ignore) {");
		src.indentPlus();
		src.println("Trace.errln(\"Failed to close the PreparedStatement (StoreData)\");");
		src.println("Trace.errln(ignore);");
		src.indentMinus();
		src.println("}");
		src.indentMinus();
		src.println("}");
		src.println("if (conn != null) {");
		src.indentPlus();
		src.println("try {");
		src.indentPlus();
		src.println("conn.close();");
		src.indentMinus();
		src.println("} catch (Exception ignore) {");
		src.indentPlus();
		src.println("Trace.errln(\"Failed to close the Connection (StoreData)\");");
		src.println("Trace.errln(ignore);");
		src.indentMinus();
		src.println("}");
		src.indentMinus();
		src.println("}");
		src.indentMinus();
		src.println("}"); // try-catch-finally
	    }
	    src.indentMinus();
	    src.println("}");
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
    }

}
